package at.ac.tuwien.dbai.alternation.compiler;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeSet;

import org.antlr.runtime.ANTLRInputStream;
import org.antlr.runtime.RecognitionException;
import org.antlr.runtime.TokenRewriteStream;
import org.antlr.stringtemplate.StringTemplateGroup;

/**
 * Gernerates Java code from an alternating algorithm
 *
 * @author Wolfgang Dvorak
 *
 */
public class atmCompile {
	public static final String USAGE = "Usage: java atmCompile ATMFILE";
	/**
	 * Takes a .atm file  and computes a java class
	 *
	 * @param inputPath path to a .atm file
	 * @param outputPath path to save the output .java file
	 * @throws IOException
	 * @throws RecognitionException
	 */
	public static void compile (String inputPath, String outputPath) throws IOException, RecognitionException{

		StringTemplateGroup templates =
			new StringTemplateGroup(
					new BufferedReader(
							new InputStreamReader(
									ClassLoader.getSystemResourceAsStream("at/ac/tuwien/dbai/alternation/compiler/atm.stg"))));
		//groupFileR .close();



		ANTLRInputStream input = new ANTLRInputStream(new  FileInputStream(inputPath));
		alternating_machineLexer lexer = new alternating_machineLexer(input);
		TokenRewriteStream tokens = new TokenRewriteStream(lexer);
		alternating_machineParser parser= new alternating_machineParser(tokens);
		parser.setTemplateLib(templates);
		parser.atm_file();

	    FileWriter  writer = new FileWriter(outputPath);
	    BufferedWriter  bWriter = new BufferedWriter(writer);
	    bWriter.write(tokens.toString());
	    bWriter.close();
	    System.out.println(outputPath +" generated");
	}

	public static void compile (InputStream inputPath, String outputPath) throws IOException, RecognitionException{

		StringTemplateGroup templates =
			new StringTemplateGroup(
					new BufferedReader(
							new InputStreamReader(
									ClassLoader.getSystemResourceAsStream("at/ac/tuwien/dbai/alternation/compiler/atm.stg"))));
		//groupFileR .close();



		ANTLRInputStream input = new ANTLRInputStream(inputPath);
		alternating_machineLexer lexer = new alternating_machineLexer(input);
		TokenRewriteStream tokens = new TokenRewriteStream(lexer);
		alternating_machineParser parser= new alternating_machineParser(tokens);
		parser.setTemplateLib(templates);
		parser.atm_file();

	    FileWriter  writer = new FileWriter(outputPath);
	    BufferedWriter  bWriter = new BufferedWriter(writer);
	    bWriter.write(tokens.toString());
	    bWriter.close();
	    System.out.println(outputPath +" generated");
	}

	public static void main(String[] args) throws IOException, RecognitionException {

		if (args.length > 0) {
			if (args.length==1) {
				int pos=args[0].lastIndexOf(".atm");
				if(pos<0){pos=args[0].lastIndexOf(".");}
				compile(args[0],args[0].substring(0,pos)+".java");
			} else {
				compile(args[0],args[1]);
			}
    	}
    	else {
			System.out.println(USAGE);
			compileExamples();
		}
	}

	public static void compileExamples() throws IOException, RecognitionException {
		System.out.println("Compile examples:");

		Set<String> examples = new TreeSet<String>();
		String path="src/at/ac/tuwien/dbai/alternation/examples/";
		String path2="at/ac/tuwien/dbai/alternation/examples/";
		String input,output;

		examples.add("GGAtm");
		examples.add("OneDimensionalCellularAutomata");
		examples.add("AlternatingTuringMachine");
		examples.add("Horn");
		examples.add("Horn2");
		examples.add("Horn3");
		examples.add("CatAndMouse");
		examples.add("CatAndMouse2");
		examples.add("CatAndMouse3");
		examples.add("LFMC");
		examples.add("MCVP");
		examples.add("PEBBLE");
		examples.add("PEBBLE2");
		examples.add("TicTacToe");
		/* Lets argue !!! */
		examples.add("BipartiteArgumentationCredulous");
		examples.add("CoBipartiteArgumentationCredulous");
		examples.add("BipartiteArgumentationSceptical");
		examples.add("BipartiteArgumentationCredulousElementary");
		examples.add("AcyclicGeographic");

		for(String example: examples){

			input=path2+example+".atm";
			output=path+example+".java";
			try{
				compile(ClassLoader.getSystemResourceAsStream(input),output);
			} catch (java.io.FileNotFoundException e){
				System.out.println(e);
			}
		}


		System.out.println("examples compiled");
	}
}
