package at.ac.tuwien.dbai.alternation.examples;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import at.ac.tuwien.dbai.alternation.runtime.ComputationNode;
import at.ac.tuwien.dbai.alternation.runtime.ComputationTree;

public class HornTest {

	private static void addRule(Map<Character,Set<Set<Character>>> rules, char head, Set<Character> body){
		if (rules.containsKey(head)){
			rules.get(head).add(body);
		} else {
			rules.put(head, new HashSet<Set<Character>>());
			rules.get(head).add(body);
		}
	}

	private static void addBinaryRule(Map<Character,Set<Set<Character>>> rules, char head, char body1, char body2){
		Set<Character> body =new HashSet<Character>();
		body.add(body1);
		body.add(body2);
		addRule(rules,head, body);
	}

	private static void addLiteral(Map<Character,Set<Set<Character>>> rules, char head){
		if (!rules.containsKey(head)){
			rules.put(head, new HashSet<Set<Character>>());
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Set<Character> facts= new HashSet<Character>();
		Map<Character,Set<Set<Character>>> rules= new HashMap<Character,Set<Set<Character>>>();
		char goal;

		/*
		facts.add('a');
		facts.add('b');
		goal='d';
		addBinaryRule(rules,'c','a','b');
		addBinaryRule(rules,'c','a','e');
		addBinaryRule(rules,'d','a','e');
		addBinaryRule(rules,'d','b','e');
		addLiteral(rules,'a');
		addLiteral(rules,'b');
		addLiteral(rules,'e');
		*/
		facts.add('a');
		facts.add('b');
		goal='g';
		addBinaryRule(rules,'c','a','b');
		addBinaryRule(rules,'e','a','c');
		addBinaryRule(rules,'f','b','e');
		addBinaryRule(rules,'g','b','f');
		addBinaryRule(rules,'g','b','f');
		addBinaryRule(rules,'g','d','a');
		addBinaryRule(rules,'g','d','b');
		addBinaryRule(rules,'g','d','c');
		addBinaryRule(rules,'g','d','e');
		addBinaryRule(rules,'g','d','f');
		addBinaryRule(rules,'c','d','a');
		addBinaryRule(rules,'e','d','a');
		addBinaryRule(rules,'f','d','b');
		addBinaryRule(rules,'f','d','e');

		//addBinaryRule(rules,'d','a','g');
		addLiteral(rules,'d');
		addLiteral(rules,'h');



		Horn atm = new Horn();
	    System.out.println(atm.compute(goal,rules,facts));
		System.out.println("#ComputationNodes: "+ atm.getComputationTree().size());

		ComputationTree<Horn.Worktape>  cT =atm.getComputationTree();
		System.out.println(cT.size());
		drawProofTree(cT);
	}

	public static void drawProofTree(ComputationTree<Horn.Worktape> cT){
		drawNode(cT,cT.getRoot());
	}

	public static void drawNode(ComputationTree<Horn.Worktape> cT, ComputationNode<Horn.Worktape> node){
		List<ComputationNode<Horn.Worktape>> childs=cT.getChildren(node);

			if (childs!=null){
				if (node.isAllQuantified()){
					System.out.println(node.getWorktape().head + ":- "+ node.getWorktape().body);
					for (ComputationNode<Horn.Worktape> child : childs){
						if (cT.get(child).equals(true)){
							drawNode(cT,child);
						}
					}
				} else {
					for (ComputationNode<Horn.Worktape> child : childs){
							drawNode(cT,child);
					}
				}
			}
	}
}
