package at.ac.tuwien.dbai.alternation.examples;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
/**
 * Transition Function delta for a nondeterministic 2-string Turing machines
 * @author Wolfgang Dvorak
 *
 */
public class TransitionFunction {
	/**
	 * An Input tuple (input symbol, worksymbol, state) for the transition function
	 * @author Wolfgang Dvorak
	 *
	 */
	static class Input{
		char inputSymbol;
		char workSymbol;
		int state;
		
		public Input(char inputSymbol, char workSymbol, int state) {
			super();
			this.inputSymbol = inputSymbol;
			this.workSymbol = workSymbol;
			this.state = state;
		}
		
		@Override
		public int hashCode() {
			final int PRIME = 31;
			int result = 1;
			result = PRIME * result + inputSymbol;
			result = PRIME * result + state;
			result = PRIME * result + workSymbol;
			return result;
		}
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			final Input other = (Input) obj;
			if (inputSymbol != other.inputSymbol)
				return false;
			if (state != other.state)
				return false;
			if (workSymbol != other.workSymbol)
				return false;
			return true;
		}

		
	}
	/**
	 * An tuple in the Output Set (work symbol, new state, inputCursor movement, workCursor movement) 
	 * @author Wolfgang Dvorak
	 *
	 */
	static class Output {
		char workSymbol;
		int newState;
		int inputCursor;
		int workCursor;
		
		public Output(char workSymbol, int newState, int inputCursor, int workCursor) {
			super();
			this.workSymbol = workSymbol;
			this.newState = newState;
			this.inputCursor = inputCursor;
			this.workCursor = workCursor;
		}

		@Override
		public int hashCode() {
			final int PRIME = 31;
			int result = 1;
			result = PRIME * result + inputCursor;
			result = PRIME * result + newState;
			result = PRIME * result + workCursor;
			result = PRIME * result + workSymbol;
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			final Output other = (Output) obj;
			if (inputCursor != other.inputCursor)
				return false;
			if (newState != other.newState)
				return false;
			if (workCursor != other.workCursor)
				return false;
			if (workSymbol != other.workSymbol)
				return false;
			return true;
		}
		
	}
	
	Map<Input,Set<Output>> function = new HashMap<Input,Set<Output>>();
	/**
	 * adds a transition rule
	 * @param inputSymbol
	 * @param workSymbol
	 * @param state
	 * @param newWorkSymbol
	 * @param newState
	 * @param inputCursor
	 * @param workCursor
	 */
	public void addRule (char inputSymbol, char workSymbol, int state, char newWorkSymbol, int newState, int inputCursor, int workCursor){
		Input input = new Input(inputSymbol,workSymbol,state); 
		Output output = new Output(newWorkSymbol,newState, inputCursor, workCursor);
		if (function.containsKey(input)){
			function.get(input).add(output);
		} else {
			function.put(input,new HashSet<Output>());
			function.get(input).add(output);
		}
	}
	/**
	 * computes the transition function
	 * @param inputSymbol
	 * @param workSymbol
	 * @param state
	 * @return the set of possible successors 
	 */
	public Set<Output> compute(char inputSymbol, char workSymbol, int state){
		Input input = new Input(inputSymbol,workSymbol,state); 
		if (function.containsKey(input)){
			return function.get(input);
		} else {
			return new HashSet<Output>();
		}
		
	}
}
