package at.ac.tuwien.dbai.alternation.examples;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
/**
 * An undirected graph
 * @author Wolfgang Dvorak
 *
 * @param <NTyp>  type of the content saved in graph nodes
 */
public class UndirectedGraph<NTyp> implements Iterable<NTyp>{
	/**
	 * A node in an undirected graph
	 * @author Wolfgang Dvorak
	 *
	 * @param <NTyp>  type of the content saved in graph nodes
	 */
	public static class Node<NTyp>{
		private NTyp object;
		private Set<Node<NTyp>> neighbours;

		public Node(NTyp object){
			this.object=object;
			this.neighbours=new HashSet<Node<NTyp>>();
		}

		public NTyp getObject(){
			return this.object;
		}

		public Set<Node<NTyp>> getNeighbours(){
			return this.neighbours;
		}

		public boolean isLinked(){
			return !this.neighbours.isEmpty() ;
		}

		public boolean addNeighbour(Node<NTyp> child){
			if(!this.neighbours.contains(child)){
				this.neighbours.add(child);
				return true;
			}else{
				return false;
			}
		}

		public boolean removeNeighbour(Node<NTyp> child){
			return this.neighbours.remove(child);
		}

		public Iterator<Node<NTyp>> neighboursIterator(){
			return this.neighbours.iterator();
		}

		public String toString() {
			return object.toString();
		}
	}
	private Map<NTyp,Node<NTyp>> nodes;

	public UndirectedGraph() {
		this.nodes=new HashMap<NTyp,Node<NTyp>>();
	}

	public Iterator<NTyp> iterator() {
		return nodes.keySet().iterator();
	}

	public boolean addNode(NTyp node){
		if (nodes.containsKey(node)){
			return false;
		}
		nodes.put(node, new Node<NTyp>(node));
		return true;
	}

	public boolean removeNode(NTyp node){
		if (nodes.containsKey(node)){
			Node<NTyp> mnode = nodes.get(node);
			nodes.remove(node);
			// remove edges
			for(Node<NTyp> mnode2 : nodes.values()){
				mnode2.removeNeighbour(mnode);
				mnode2.removeNeighbour(mnode);
			}
			return true;
		}
		return false;
	}

	public boolean addEdge(NTyp node1,NTyp node2){
		if (nodes.containsKey(node1) & nodes.containsKey(node2)){
			nodes.get(node1).addNeighbour(nodes.get(node2));
			nodes.get(node2).addNeighbour(nodes.get(node1));
			return true;
		} else {
			return false;
		}
	}

	public boolean removeEdge(NTyp node1,NTyp node2){
		if (nodes.containsKey(node1) & nodes.containsKey(node2)){
			return nodes.get(node1).removeNeighbour(nodes.get(node2)) &
			nodes.get(node2).removeNeighbour(nodes.get(node1));
		} else {
			return false;
		}
	}

	public Node<NTyp> get(NTyp position) {
		return nodes.get(position);
	}

	public int size() {
		return nodes.size();
	}
}
