package at.ac.tuwien.dbai.alternation.runtime;
/**
 * Node in the ComputionTree
 * holds:
 *  # the state of the machine
 *  # the worktape with the work variables
 *  # the Quantifier associed with the node - unset if halting ( immediate accepting / rejecting) node
 *
 * @author Wolfgang Dvorak
 *
 * @param <GWorktape> container class for the work variables
 */
public class ComputationNode<GWorktape extends InterfaceWorktape> {
	/**
	 * true iff universal quantified
	 * false iff existential quantified
	 *
	 * unset if halting state
	 */
	Boolean allQuantified;

	/**
	 * the id of the state
	 */
	byte state;

	/**
	 * container with work variables
	 */
	GWorktape worktape;

	/**
	 *
	 * @param state
	 * @param worktape
	 */
	public ComputationNode (byte state, GWorktape worktape) {
		super();
		this.worktape = (GWorktape) worktape.clone();
		this.state=state;
	}

	/**
	 *
	 * @param forallState true for a universal quantifier and false for a existential Quantifier
	 */
	public void setQuantifier(boolean forallState) {
		this.allQuantified=new Boolean(forallState);
	}

	public String toString() {
		if (allQuantified==null)
		{
			return "[, State " + state +", "+worktape.toString()+"]";
		}
		if (allQuantified) {
			return "[ForallState, State " + state +", "+worktape.toString()+"]";
		} else {
			return "[ExistsState, State " + state +", "+worktape.toString()+"]";
		}

	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((worktape == null) ? 0 : worktape.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final ComputationNode<GWorktape>  other = (ComputationNode<GWorktape>) obj;
		if (state != other.state)
			return false;
		if (worktape == null) {
			if (other.worktape != null)
				return false;
		} else if (!worktape.equals(other.worktape))
			return false;
		return true;
	}

	/**
	 *
	 * @return true if universal quantified, false if existential quantified otherwise null
	 */
	public Boolean isAllQuantified() {
		if(allQuantified==null) return null;
		else if (allQuantified.equals(true)) return true;
		return false;
	}

	/**
	 *
	 * @return the id of the state
	 */
	public byte getState() {
		return state;
	}
	/**
	 *
	 * @return container with work variables
	 */
	public GWorktape getWorktape() {
		return worktape;
	}
}