package at.ac.tuwien.dbai.alternation.runtime;

import java.util.ArrayList;
import java.util.List;

/**
 * Result of a state-computation on a worktape. 
 * So it saves if a configuration immediate accepts / rejects 
 * or its successors and if it is a universal or existential quantified configuration 
 * 
 * @author Wolfgang Dvorak
 *
 * @param <GWorktape> container for workvariables
 */
public class ResultTuple<GWorktape> {

	/**
	 * simple container to save pairs of states and worktapes
	 * @author Wolfgang Dvorak
	 *
	 * @param <GWorktape> container for workvariables
	 */
	protected static class AtmConfiguration<GWorktape> {
		public byte state;
		public GWorktape worktape;

		public AtmConfiguration(byte state, GWorktape worktape) {
			this.state=state;
			this.worktape=worktape;
		}

	}

	/**
	 * ResultTuple the means a computation accepts
	 */
	public static final ResultTuple ACCEPT = new ResultTuple(true,false,false);
	/**
	 * ResultTuple the means a computation rejects
	 */
	public static final ResultTuple REJECT = new ResultTuple(false,true,false);
	
	protected boolean accept;
	protected boolean reject;
	protected boolean forall;
	protected List<AtmConfiguration<GWorktape>> successors;

	/**
	 * Constructor
	 * @param accept true if the computation accepts
	 * @param reject true if the computation rejects
	 * @param forall if unversal quantified, false if existential quantified
	 */
	public ResultTuple(boolean accept, boolean reject, boolean forall) {
		super();
		this.accept = accept;
		this.reject = reject;
		this.forall = forall;
		this.successors = new ArrayList<AtmConfiguration<GWorktape>>();
	}
	
	/**
	 * adds a new Configuration to the Successors
	 * @param state stateId
	 * @param worktape worktape
	 */
	public void add(byte state, GWorktape worktape){
		successors.add(new AtmConfiguration<GWorktape>(state,worktape));
	}
}